/*-
 * Copyright (c) 2002  Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: commandline.c,v 0.94 2005/10/04 12:11:54 shelton Exp $
 */
/*------------------------------------------------------------------------

        Spam Control and Canceling mail filter, based on libmilter API

 	UNIX FreeBSD 5.4 version

        Commandline processing.

        Call: void _SCC_parse_commandline(int argc, char *argv[])

	No return when error detected.

--------------------------------------------------------------------------*/
#include "sccmilter.h"
#include "externals.h"
#include "functions.h"

// Main program section

void _SCC_milter_parse_commandline(int argc, char *argv[])
{
  extern char *optarg;

  char c;

  const char *hvalid  = "-l (--html) should contain valid HTML processing mode";
  const char *specmode =
              "mode must be specified as \"reject\", \"discard\" or \"isolate\"";

  const char *options = "a:hvdc:D:p:t:m:ne:l:g:P:o:~`^";
  const char *mustbe = "must be specified";

  static const struct option long_options[] =
   {
     {"help",                   0, 0, 'h' },
     {"version",                0, 0, 'v' },
     {"daemon",                 0, 0, 'd' },
     {"sendmail",               1, 0, 'c' },
     {"debug",                  1, 0, 'D' },
     {"tmpdir",                 1, 0, 'p' },
     {"extrasave",              1, 0, 'a' },
     {"sendmail-timeout",       1, 0, 't' },
     {"mode",	           	1, 0, 'm' },
     {"enterprise",		0, 0, 'n' },
     {"emode",			1, 0, 'e' },
     {"nohtml",			1, 0, 'l' },
     {"codepage",		1, 0, 'g' },
     {"pidfile",		1, 0, 'P' },
     {"domain",			1, 0, 'o' },
     {"imitate",		0, 0, '~' },
     {"no-delete",		0, 0, '`' },
     {"spam-no-delete",		0, 0, '^' },
     { NULL,                    0, 0,  0  }
   };

// Process short or long options

  while ((c = getopt_long(argc, argv, options, long_options, NULL)) != EOF)
   {
     switch (c)
      {
        case 'c': // Sendmail communication socket
         if (*optarg == '\0')
           {
             fprintf(stderr,"sendmail communication socket %s\n",mustbe);
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.sendmail_pipe,optarg);
         break;

        case 't': // Sendmail connect timeout
         if (*optarg == '\0')
           {
             fprintf(stderr,"sendmail connect timeout %s\n",mustbe);
             exit(EX_USAGE);
           }

         _SCC_milter_config.sendmail_timeout = atoi(optarg);
         break;

        case 'p': // Temp directory
         if (*optarg == '\0')
           {
             fprintf(stderr,"temp directory %s\n",mustbe);
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.isolation,optarg);
         break;

        case 'a': // Extra save directory
         if (*optarg == '\0')
           {
             fprintf(stderr,"extra save directory %s\n",mustbe);
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.extrasave,optarg);
         break;

        case 'P': // PID file specification
         if (*optarg == '\0')
           {
             fprintf(stderr,"PID file directory and name %s\n",mustbe);
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.pidfile,optarg);
         break;

        case 'o': // Local domain specification
         if (*optarg == '\0')
           {
             fprintf(stderr,"Local domain parameter %s\n",mustbe);
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.domain,optarg);
         break;

        case 'g': // Allowed charsets list
         if (*optarg == '\0')
           {
             fprintf(stderr,"-g (--codepage) should contain at least one charset\n");
             exit(EX_USAGE);
           }

         strcpy(_SCC_milter_config.codepages,optarg);
         break;

        case 'd': // Daemon mode
         _SCC_milter_config._idflags._isdaemon = OK;
         break;

        case '~': // Imitate mode
         _flags._imitate = OK;
         break;

        case '`': // Any mail no deletion mode
         _flags._nodelete = OK;
         break;

        case '^': // Spam mail no deletion mode
         _flags._spam_nodelete = OK;
         break;

        case 'l': // HTML isolation
         if (!_checklist(optarg,(char *) &_SCC_milter_config.hmode,&html_modes))
           {
             fprintf(stderr,"%s\n",hvalid);
             exit(EX_USAGE);
           }

         break;

        case 'n': 		// Enterprise violation checks activating
         _SCC_milter_config._idflags._enterprise = OK;
         break;

        case 'D': 		// Debug level
         if (atoi(optarg) < 0)
           {
             fprintf(stderr,"invalid debug level value, must be between 0 - 255\n");
             exit(EX_USAGE);
           }

         _SCC_milter_config.debug_level = atoi(optarg);
         break;

        case 'm': 		// Mode for spam messages
         if (!_checklist(optarg,(char *) &_SCC_milter_config.mode,&act_modes))
           {
             fprintf(stderr,"%s\n",specmode);
             exit(EX_USAGE);
           }

         break;

        case 'e': 		// Mode for enterprise violated messages
         if (!_checklist(optarg,(char *) &_SCC_milter_config.emode,&act_modes))
           {
             fprintf(stderr,"%s\n",specmode);
             exit(EX_USAGE);
           }

         break;

        case 'v': 		// Version info requested
         fprintf(stderr,"Package: %s-%s\n%s\n%s, version %s\n",
             			PKGNAME,PKGVERSION,SCC_MILTER,SCC_DATE,PKGVERSION);
         exit(EX_USAGE);

        case 'h': 		// Help requested or unknown parameter was given
        default:
         _SCC_milter_usage();
         exit(EX_USAGE);
      }
   }

  return;
}
