/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: filecopy.c,v 0.94.2.1 2005/10/06 07:08:22 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

            Issue copying of file

            FreeBSD 5.4 version

            Call: int __filecopy(void *infile, void *outfile,
            			  mode_t filemode, int append, int fcbs, int fcbd);

            Call with: infile - pointer to source path or FCB
            	       outfile - pointer to destination path or FCB
                       filemode - mode, which will be applied to destination
                       append - see below
                       fcbs - see below
                       fcbd - see below

            When append is FC_COPY, file will simply copied
            When append is FC_APPEND, file will append to outfile

            When fcbs is FB_SOURCE, source file will open by path
            When fcbs is FB_FCB, source file will already opened
            			and it is FILE *pointer

            When fcbd is FB_SOURCE, dest file will open by path
            When fcbd is FB_FCB, dest file will already opened
            			and it is FILE *pointer

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
#include "make.h"

int __filecopy(void *infile, void *outfile, mode_t filemode, int append, int fcbs, int fcbd)
{
  int i, fmode;
  short int handle,ohandle,k;
  char *array;
  struct stat sb;

// Check mode, which was called

  if (fcbs == FB_FCB)
    {
      rewind((FILE *) infile);
      handle = fileno((FILE *) infile);

      fstat(handle, &sb);
    }
   else
     {

// Check on opening source file, fail when error

       if ((handle = open((char *) infile, O_RDONLY)) < NULL)
         return ERR;

       stat((char *) infile, &sb);
     }

  array = (char *) malloc(1024);

// Try to open output file, when source mode, else get handle

  if (fcbd == FB_SOURCE)
    {

// Create file open mode (depend from append parameter)

      if (append)
        fmode = O_WRONLY | O_APPEND | O_EXLOCK;
       else
         fmode = O_CREAT | O_TRUNC | O_WRONLY | O_EXLOCK;

// Open output file

      if ((ohandle = open((char *) outfile, fmode)) < NULL)
        {
          free(array);
          return OK;
        }
    }
   else
     {
       ohandle = fileno((FILE *) outfile);

// Seek to EOF, when append mode is on

       if (append)
         fseek((FILE *) outfile, 0L, SEEK_END);
     }

// Moving content with blocks by 1024 bytes

  for(i = 0;i < sb.st_size;i += k)
   {
     k = read(handle, array, 1024);
     write(ohandle, array, k);
   }

// Close handles and set mode. Input handle closing only FB_SOURCE

  if (fcbs == FB_SOURCE)
    close(handle);

  if (fcbd == FB_SOURCE)
    {
      close(ohandle);
      chmod(outfile,filemode);
    }
   else
     fchmod(ohandle,filemode);

  free(array);
  return NULL;
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from source path to another source path

            FreeBSD 5.4 version

            Call: int fsscopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fsscopy(char *infile, char *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_COPY, FB_SOURCE, FB_SOURCE);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from source path to another source path with append
            to source file

            FreeBSD 5.4 version

            Call: int fssacopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fssacopy(char *infile, char *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_APPEND, FB_SOURCE, FB_SOURCE);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from file descriptor to another source path with append
            to source file

            FreeBSD 5.4 version

            Call: int fbsacopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fbsacopy(FILE *infile, char *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_APPEND, FB_FCB, FB_SOURCE);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from file descriptor to another source path

            FreeBSD 5.4 version

            Call: int fbscopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fbscopy(FILE *infile, char *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_COPY, FB_FCB, FB_SOURCE);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from a source path to file descriptor

            FreeBSD 5.4 version

            Call: int fsbcopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fsbcopy(char *infile, FILE *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_COPY, FB_SOURCE, FB_FCB);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from a source path to file descriptor with append
            to tail of source file

            FreeBSD 5.4 version

            Call: int fsbacopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fsbacopy(char *infile, FILE *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_APPEND, FB_SOURCE, FB_FCB);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from file descriptor to file descriptor with append
            to tail of source file

            FreeBSD 5.4 version

            Call: int fbbacopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fbbacopy(FILE *infile, FILE *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_APPEND, FB_FCB, FB_FCB);
}

/*-------------------------------------------------------------------------

            Standard functions library

            Copy a file from file descriptor to file descriptor

            FreeBSD 5.4 version

            Call: int fbbcopy(char *infile, char *outfile, mode_t filemode);

            Return: 0  - file was succesfully copied/appended
            	    -1 - Errors when copying, associated with source file
                    >0 - Errors when copying, associated with destination file

-------------------------------------------------------------------------*/
int fbbcopy(FILE *infile, FILE *outfile, mode_t filemode)
{
  return __filecopy((void *) infile, (void *) outfile,
  				filemode, FC_COPY, FB_FCB, FB_FCB);
}
